Option Explicit

Global Const FormTitle = "Editor #"

Const MAX_SIZE = 3

'These came over from the child.
Global Const NO_QUERY = False
Global Const DO_QUERY = True

'This is an array of references to child objects.
'This array will be sized and expanded at run time.
Global Editors() As frmMDIChild

'This variable points to the empty spot in the Editors() array.
Global CurrentEditor As Integer

'This variable tracks the current size of the array Editor()
'and the array EditorData().
Dim EditorCount As Integer

'Each child that's created will need these 2 variables.
Type EdRecord
   dirtyflag As Integer
   filename As String
End Type

'This array, which expands as needed, holds each child's copy of the
'Type defined above.
Global EditorData() As EdRecord

'Arrange Method for MDI Forms
Global Const CASCADE = 0
Global Const TILE_HORIZONTAL = 1
Global Const TILE_VERTICAL = 2
Global Const ARRANGE_ICONS = 3

'WindowState
Global Const NORMAL = 0
Global Const MINIMIZED = 1
Global Const MAXIMIZED = 2

'Clipboard
Global Const CF_TEXT = 1

'MsgBox parameters
Global Const MB_OK = 0
Global Const MB_OKCANCEL = 1
Global Const MB_ABORTRETRYIGNORE = 2
Global Const MB_YESNOCANCEL = 3
Global Const MB_YESNO = 4
Global Const MB_RETRYCANCEL = 5

Global Const MB_ICONSTOP = 16
Global Const MB_ICONQUESTION = 32
Global Const MB_ICONEXCLAMATION = 48
Global Const MB_ICONINFORMATION = 64

Global Const MB_APPLMODAL = 0
Global Const MB_DEFBUTTON1 = 0
Global Const MB_DEFBUTTON2 = 256
Global Const MB_DEFBUTTON3 = 512
Global Const MB_SYSTEMMODAL = 4096

'MsgBox return values
Global Const IDOK = 1
Global Const IDCANCEL = 2
Global Const IDABORT = 3
Global Const IDRETRY = 4
Global Const IDIGNORE = 5
Global Const IDYES = 6
Global Const IDNO = 7

Sub CreateNewEditor (sfilename As String)
    Dim fhandle As Integer

    'For the very first Editor child, it's necessary to size the 2 arrays.
    'No new child is needed: used the one obtained at design time.
    If CurrentEditor = 0 Then
	CurrentEditor = 1
	EditorCount = MAX_SIZE
	ReDim Editors(EditorCount)
	ReDim EditorData(EditorCount)
	Set Editors(CurrentEditor) = frmMDIChild
    Else
	'In all other cases, create a new instance of an Editor child.
	Dim newEditor As New frmMDIChild
	'Next, determine if the arrays need to be expanded.
	If CurrentEditor > EditorCount Then
	    EditorCount = EditorCount + MAX_SIZE
	    ReDim Preserve Editors(EditorCount)
	    ReDim Preserve EditorData(EditorCount)
	End If
	'Finally, 'point' to the newest Editor.
	Set Editors(CurrentEditor) = newEditor
    End If

    'If the incoming argument is not empty, open the file
    'and load it into the current child's text box.
    If sfilename <> "" Then
	fhandle = FreeFile
	Open sfilename For Input As fhandle
	Editors(CurrentEditor).txtEditBox.Text = Input$(LOF(fhandle), fhandle)
	'Build the title without a path.
	Editors(CurrentEditor).Caption = FormTitle + Str$(CurrentEditor) + " (" + sfilename + ")"
	Close fhandle
    Else
	Editors(CurrentEditor).Caption = FormTitle & CurrentEditor
    End If
	
    'Set the 2 variables in the array.
    EditorData(CurrentEditor).filename = sfilename
    EditorData(CurrentEditor).dirtyflag = False

    'Create a tag that will be the child's number, and show the child.
    Editors(CurrentEditor).Tag = Str$(CurrentEditor)
    Editors(CurrentEditor).Show
    
    'Finally, bump the counter for the next child.
    CurrentEditor = CurrentEditor + 1
End Sub

Function GetFileName () As String
    'Set up the common dialog box.
    frmMDIParent.CMDialog1.Filter = "Text Files (*.txt)|*.txt|All files (*.*)|*.*"
    frmMDIParent.CMDialog1.filename = ""

    'Invoke the Dialog Box...
    frmMDIParent.CMDialog1.Action = 1

    'and return the filename.
    GetFileName = frmMDIParent.CMDialog1.filename
End Function

Sub SaveCurrentEditor (query As Integer)
    Dim fhandle As Integer
    Dim i As Integer, rc As Integer
    Dim fname As String

    'Use a different method to obtain the tag
    i = Val(frmMDIParent.ActiveForm.Tag)

    'There's nothing to do if the dirty flag isn't set!
    If EditorData(i).dirtyflag = False Then
	Exit Sub
    End If

    'Only query if the incoming argument requires it.
    If (query = True) Then
	
	'Build an appropriate string for the message box.
	If EditorData(i).filename = "" Then
	    fname = "Buffer"
	Else
	    fname = EditorData(i).filename
	End If
	
	'Determine what the user wants to do.
	rc = MsgBox(fname + " has changed. Save before terminating?", MB_YESNO)
	If rc = IDNO Then
	    Exit Sub
	End If
    End If
    
    'If the user just opened a new window text without opening a file, then
    'there's no file name, so one must be obtained.
    If EditorData(i).filename = "" Then
	EditorData(i).filename = InputBox$("Enter a file name. File will be saved in " + CurDir$, "No file name available!")
	frmMDIParent.ActiveForm.Caption = FormTitle & frmMDIParent.ActiveForm.Tag + " (" + EditorData(i).filename + ")"
    End If
    
    'Now if the filename is still empty, then just end this subroutine.
    If EditorData(i).filename = "" Then
	Exit Sub
    End If

    'Get a fresh handle and write the file to it.
    fhandle = FreeFile
    Open EditorData(i).filename For Output As fhandle
    Print #fhandle, frmMDIParent.ActiveForm.txtEditBox.Text
    Close fhandle
    EditorData(i).dirtyflag = False
End Sub

